/* filtersidebar.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include "filtersidebar.h"
#include "filterdetails.h"
#include "main.h"
#include "gui.h"
#include "filter.h"
#include "session.h"
#include "data.h"
#include "device.h"

/***************************************************************************
 Callbacks
 ***************************************************************************/

static void select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((FilterSideBar*)data)->select(row); }
static void unselect_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((FilterSideBar*)data)->unselect(); }
static void new_cb(GtkWidget *w, gpointer data)
{ ((FilterSideBar*)data)->newlist(); }
static void edit_cb(GtkWidget *w, gpointer data)
{ ((FilterSideBar*)data)->edit(); }
static void apply_cb(GtkWidget *w, gpointer data)
{ ((FilterSideBar*)data)->apply(); }
static void delete_cb(GtkWidget *w, gpointer data)
{ ((FilterSideBar*)data)->deletelist(); }

/***************************************************************************
 FilterSideBar Class
 ***************************************************************************/

FilterSideBar::FilterSideBar(Main *m)
{
	GtkWidget *label183;
	GtkWidget *scrolledwindow29;
	GtkWidget *hbox89;
	GtkWidget *hbox90;

	main = m;
	row = -1;

  vbox56 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox56);
  gtk_container_set_border_width (GTK_CONTAINER (vbox56), 2);

  label183 = gtk_label_new ("View Filters");
  gtk_widget_show (label183);
  gtk_box_pack_start (GTK_BOX (vbox56), label183, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label183), 0, 0.5);

  scrolledwindow29 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow29);
  gtk_box_pack_start (GTK_BOX (vbox56), scrolledwindow29, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow29),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  clist26 = gtk_clist_new (1);
  gtk_widget_show (clist26);
  gtk_container_add (GTK_CONTAINER (scrolledwindow29), clist26);
  gtk_widget_set_usize (clist26, 160, -2);
  gtk_clist_set_column_width (GTK_CLIST (clist26), 0, 16);
  gtk_clist_column_titles_hide (GTK_CLIST (clist26));
	gtk_signal_connect(GTK_OBJECT(clist26), "select_row",
		GTK_SIGNAL_FUNC(select_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist26), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_cb), this);

  hbox89 = gtk_hbox_new (TRUE, 4);
  gtk_widget_show (hbox89);
  gtk_box_pack_start (GTK_BOX (vbox56), hbox89, FALSE, TRUE, 0);

  button95 = gtk_button_new_with_label ("New");
  gtk_widget_show (button95);
  gtk_box_pack_start (GTK_BOX (hbox89), button95, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button95), "clicked",
		GTK_SIGNAL_FUNC (new_cb), this);

  button96 = gtk_button_new_with_label ("Edit");
	gtk_widget_show (button96);
  gtk_box_pack_start (GTK_BOX (hbox89), button96, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button96), "clicked",
		GTK_SIGNAL_FUNC (edit_cb), this);

  hbox90 = gtk_hbox_new (TRUE, 4);
  gtk_widget_show (hbox90);
  gtk_box_pack_start (GTK_BOX (vbox56), hbox90, FALSE, TRUE, 0);

  button97 = gtk_button_new_with_label ("Apply");
  gtk_widget_show (button97);
  gtk_box_pack_start (GTK_BOX (hbox90), button97, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button97), "clicked",
		GTK_SIGNAL_FUNC (apply_cb), this);

  button98 = gtk_button_new_with_label ("Delete");
	gtk_widget_show (button98);
  gtk_box_pack_start (GTK_BOX (hbox90), button98, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button98), "clicked",
		GTK_SIGNAL_FUNC (delete_cb), this);
}

FilterSideBar::~FilterSideBar()
{
}

void FilterSideBar::select(int r)
{
	row = r;
	gtk_widget_set_sensitive(button96, TRUE);
	gtk_widget_set_sensitive(button97, TRUE);
	gtk_widget_set_sensitive(button98, TRUE);
}

void FilterSideBar::unselect()
{
	gtk_widget_set_sensitive(button96, FALSE);
	gtk_widget_set_sensitive(button97, FALSE);
	gtk_widget_set_sensitive(button98, FALSE);
	row = -1;
}

void FilterSideBar::newlist()
{
	Filter f;
	main->gui->filterdetails->orig = NULL;
	main->gui->filterdetails->copytowidgets(&f);
	main->gui->filterdetails->show();
}

void FilterSideBar::edit()
{
	Filter *f = (Filter*) gtk_clist_get_row_data(GTK_CLIST(clist26), row);
	main->gui->filterdetails->orig = f;
	main->gui->filterdetails->copytowidgets(f);
	main->gui->filterdetails->show();
}

void FilterSideBar::apply()
{
	Link *ln;
	Filter *f;
	Device *d;
	Network *n;

	f = (Filter*) gtk_clist_get_row_data(GTK_CLIST(clist26), row);

	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		d->matchfilter(f);
	}
	for (ln=main->session->data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->matchfilter(f);
	}
	if (!main->session->positionbit)
	{
		main->session->positionbit=TRUE;
		main->setTitle();
	}
	main->gui->update_views();
}

void FilterSideBar::deletelist()
{
	Filter *f;

	f = (Filter*) gtk_clist_get_row_data(GTK_CLIST(clist26), row);
	main->session->filters.del(f);
	if (!main->session->filterbit)
	{
		main->session->filterbit=TRUE;
		main->setTitle();
	}
	delete f;
	update();
}

void FilterSideBar::update()
{
	Link *ln;
	Filter *f;
	gchar *data[1];
	int r = 0;

	gtk_clist_freeze(GTK_CLIST(clist26));
	gtk_clist_clear(GTK_CLIST(clist26));
	for (ln=main->session->filters.get_head(); ln!=NULL; ln=ln->next)
	{
		f = (Filter*) ln->item;
		data[0] = strdup(f->name);
		gtk_clist_append(GTK_CLIST(clist26), data);
		delete data[0];
		gtk_clist_set_row_data(GTK_CLIST(clist26), r, f); r++;
	}
	gtk_widget_set_sensitive(button96, FALSE);
	gtk_widget_set_sensitive(button97, FALSE);
	gtk_widget_set_sensitive(button98, FALSE);
	gtk_clist_set_sort_column (GTK_CLIST (clist26), 0);
	gtk_clist_sort (GTK_CLIST (clist26));
	gtk_clist_thaw(GTK_CLIST(clist26));
}

