/* topology.h
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef TOPOLOGY_H
#define TOPOLOGY_H 1

#include "ipaddress.h"
#include "list.h"

#define BUFSIZE 256

class Main;
class Subnet;
class Device;
class Network;
class Parser;
class Interface;
class Topology
{
	public:
		Topology(Main *m);
		~Topology();

		void create_placeholder(char *ipstr);
		void parse(char *fname, int hop);
		void copyDeviceHeader();
		void clearDeviceTags();
		void checkForDuplicateDevice();
		void checkForDuplicateDeviceWithIP(IPAddress *ip);
		void joinLayers();
		void mergeType();
		void attachInterfaces();
		void severNetwork(Network *nw);
		Interface *lookupInterface(Device *d, Interface *in);
		void layer3(int hop);
		void add_subnet(Subnet *s);
		void add_layer3(Network *n, IPAddress ip, int hop);
		void merge_networks(Network *n1, Network *n2);
		void layer2();
		void layer2_check(Interface *in);
		void checkForFalseMachines();
		void checkForFalseMachinesWithIP(IPAddress *ip);
		void copyOverMachines();
		void deleteTaggedDevices();

		Main *main;
		Parser *parser;
		Device *tmpdev;
		Device *origdev;
		int merge_type;
};

#endif

