/* graph.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include "graph.h"
#include "misc.h"

unsigned long get_value(unsigned long oval, unsigned long nval, int time_diff, int rate)
{
	unsigned long tmp;

	if (oval > nval)
		tmp = 0xffffffff - (oval - nval) + 1;
	else
		tmp = nval - oval;
	if (time_diff != 0)
	{
		if (rate==1)
			tmp = tmp/(unsigned long)time_diff;
		else
		if (rate==2)	// 5 minute rate
			tmp = (tmp*300)/(unsigned long)time_diff;
	}else
		tmp = 0;
	return tmp;
}

void Dvice::clear()
{
/*
	Link *ln;

	grabtime.empty();
	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		iface = (Iface*) ln->item;
		delete iface;
	}
	interfaces.empty();
*/
}

Graph::Graph()
{
}

Graph::~Graph()
{
}

// we load the current data and the old data files
// store in two device objects
// if an entry is missing from either leave data value in output as 0
// if status of either is down don't record data
// if both entries the output value is the difference (fix rounding errors)
// move current data overwriting old data

void Graph::main(char *datadir, char *device)
{
	char buf[256];
	int a, b;

	gio.read_global();
	strcpy(name, device);
	sprintf(buf, "%s/raw/%s", datadir, device);
	a = load(&d1, buf);
	sprintf(buf, "%s/raw/%s.old", datadir, device);
	b = load(&d2, buf);
	
	if ((a==1) && (b==1))
	{
printf("GOT BOTH FILES %s\n", device);
//		print();

if (strcmp(device, "x.x.x.x")==0)
{
	printf("New Time = %s\n", d1.grabtime.date_to_str(1));
	printf("Old Time = %s\n", d2.grabtime.date_to_str(1));
}

		save(datadir, device);
	}
}

int Graph::load(Dvice *dv, char *fname)
{
	FILE *f;
	char buf[256], var[256], val[256];
	int i;

//	printf("Loading %s\n", fname);
	f = fopen(fname, "r");
	if (!f) return 0;

	dv->clear();

	fscanf(f, "%[^\n]\n", buf);
	dv->grabtime.setstring(buf);
	while(fscanf(f, "%[^\n]\n", buf)==1)
	{
		if (sscanf(buf, "%[^ ] %[^\n]", var, val)==2)
		{
			for (i=0; i<gio.num_global_values; i++)
			{
				if (starts_with(gio.global_types[i].oid, var))
				{
					add_to_interface(var, val, dv, i);
				}
			}
		}
	}

	if (dv->grabtime.zero()) return 0;

	fclose(f);
	return 1;
}

void Graph::add_to_interface(char *var, char *val, Dvice *dv, int pos)
{
	char vartype[256];
	unsigned long s;
	int ifnum;
	Iface *iface, *iffound;
	Link *ln;

// printf("ADD TO INTERFACE\n%s\n%s\n", var, gio.global_types[pos].oid);

	// start by extracting the interface number
	strcpy(vartype, var+strlen(gio.global_types[pos].oid)+1);
	if (sscanf(vartype, "%d", &ifnum)!=1) return;

	// find an interface with same ifnum
	iffound = NULL;
	for (ln=dv->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		iface = (Iface*) ln->item;
		if (iface->index == ifnum) iffound = iface;
	}
	if (iffound == NULL)
	{
		iface = new Iface();
		iface->index = ifnum;
		dv->interfaces.tag(iface);
	}else
		iface = iffound;

	// handle if status value else
	if (sscanf(val, "%lu", &s)==1)
		iface->values[pos] = s;
}

void Graph::print()
{
	int i, j;
	Iface *if1, *if2;

	printf("\n\nDevice %s\n", name);
	for (i=0; i<d1.interfaces.size(); i++)
	{
		if1 = (Iface*) d1.interfaces.get_item(i);
		if2 = (Iface*) d2.interfaces.get_item(i);
		printf("Index %8d\n", if1->index);
		for (j=0; j<gio.num_global_values; j++)
		{
			printf("%20s Old %12lu, ", gio.global_types[j].label, if2->values[j]);
			printf("New %12lu\n", if1->values[j]);
		}
	}
}

void Graph::save(char *datadir, char *device)
{
	struct stat statbuf;
	Link *ln, *ln2;
	Iface *i1, *i2;
	char buf[256];
	int i, found;
	TimeStamp time_diff;

	time_diff.ts = d1.grabtime.compare(d2.grabtime);

	for (ln=d1.interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		i1 = (Iface*) ln->item;
		found = 0;
		gio.current = d1.grabtime;
//		gio.status = ???;
		for (ln2=d2.interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			i2 = (Iface*) ln2->item;
			if (i1->index == i2->index)
			{
				// fill in stuff here for this interface
				for (i=0; i<gio.num_global_values; i++)
					gio.value[i] = get_value(i2->values[i], i1->values[i],
						time_diff.ts, gio.global_types[i].rate);
printf("VALUE\n");
				found = 1;
			}
		}
		if (found)
		{
			sprintf(buf, "%s/graph", datadir);
			if (stat(buf, &statbuf) == -1) mkdir(buf, 493); // rwxr xr x
			sprintf(buf, "%s/graph/%s", datadir, device);
			if (stat(buf, &statbuf) == -1) mkdir(buf, 493); // rwxr xr x
			sprintf(buf, "%s/graph/%s/%d", datadir, device, i1->index);
			if ((stat(buf, &statbuf) == -1) || (statbuf.st_size == 0))
			{
				// no such file so create a new empty file
				gio.create(buf);
printf("CREATE\n");
			}

			gio.add(buf);
printf("ADD\n");
			// the gio values for add will be different now if the add had to
			// go into a second buffer, so they should be reset before getting
			// back here again (which they are)
		}
	}
}

